package de.devbliss.paulparser;

import java.util.List;
import java.util.regex.Pattern;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;

/**
 * This parser reads a html document and extracts publication specific information.
 * Currently this parser supports:
 * - meta keywords / description
 * - CITATION (eg. "citation_title")
 * - DC Tags (eg. "dc.title")
 * - Prism (eg. "prism.issn").
 * 
 *
 */
public class PublicationMetaTagParser {
	
	
	/**
	 * Convenience method for getting clean publication metadata
	 * from a webpage.
	 * 
	 * Parses publication data from a page and returns an integrated
	 * PublicationModel. All information is therefore de-duplicated and
	 * cleaned. 
	 * 
	 * If you want the raw data please use method parsePublicationDataRaw
	 * instead.
	 * 
	 * @param htmlContent
	 * @return
	 */
	public static PublicationModel parsePublicationData(String htmlContent) {
		
		PublicationModelRaw publicationModelRaw = parsePublicationDataRaw(htmlContent);
		
		PublicationModel publicationModel = new PublicationModel(publicationModelRaw);
		
		return publicationModel;
		
		
	}
	

	/**
	 * Parses a webpage and extracts all metadata (eg. Prism, DC tags, citation_*).
	 * 
	 * Returns a raw model containing all information retrievable. This raw model
	 * contains duplicated and non integrated content.
	 * 
	 * You may want to use PublicationModel or method parsePublicationData for convenience.
	 * 
	 * @param htmlContent
	 * @return
	 */
	public static PublicationModelRaw parsePublicationDataRaw(
			String htmlContent) {
		Document document = Jsoup.parse(htmlContent);

		PublicationModelRaw publicationParsingMetaModel = new PublicationModelRaw();

		for (MetaTags tagName : MetaTags.values()) {

			List<Element> metaTags = retrieveAllMetaTagsWithName(document, tagName.getTagName());

			for (Element metaTag : metaTags) {
				
				// Get content of "content" or "contents" attribute.
				// Usually "content" is correct.
				// but some journals are using "contents" (what is most likely wrong)
				// we accept both to fix that.				
				String contentAttribute = metaTag.attr("content");
				
				if (contentAttribute.equals("")) {
					contentAttribute = metaTag.attr("contents");
				}
				
				// Only assign the value if we got a meaningful content. 
				// Otherwise just skip that one...
				if (!contentAttribute.equals("")) {					
					// Yes. we trim the attribute. That is also often wrong
					// in the documents we get.
					contentAttribute = contentAttribute.trim();

					assignValue(publicationParsingMetaModel, tagName,
						contentAttribute);
				}
			}
		}

		return publicationParsingMetaModel;
	}

	private static List<Element> retrieveAllMetaTagsWithName(Document document,
			String metaTagName) {

		// 1. make sure to escape "." such as in "dc.title" so that it is
		//    not evaluated in regex parsing.
		// 2. Also make pattern case insensitive
		String regex = metaTagName.replaceAll("\\.", "\\\\.");
		// \\A AND \\Z makes the regex match only exactly this name.
		// Otherwise the regex would match parts of other similar tagnames
		//For instance "dc.Rights" would match "dc.RightsAgent" without 
		// \\A AND \\Z
		Pattern pattern = Pattern.compile("\\A" + regex + "\\Z",
				Pattern.CASE_INSENSITIVE);

		return document.getElementsByAttributeValueMatching("name", pattern);
	}

	private static void assignValue(
			PublicationModelRaw publicationParsingMetaModel,
			MetaTags tag, String content) {

		//////////////////////////////////////////////////////////////////////
		// General keywords / description tags of html document.
		//////////////////////////////////////////////////////////////////////
		if (tag.equals(MetaTags.KEYWORDS)) {
			publicationParsingMetaModel.keywords = content;

		} else if (tag.equals(MetaTags.DESCRIPTION)) {
			publicationParsingMetaModel.description = content;

		//////////////////////////////////////////////////////////////////////
		// Regular citation (Highwire) tags
		//////////////////////////////////////////////////////////////////////
		} else if (tag.equals(MetaTags.CITATION_TITLE)) {
			publicationParsingMetaModel.citation_title = content;

		} else if (tag.equals(MetaTags.CITATION_ABSTRACT_HTML_URL)) {
			publicationParsingMetaModel.citation_abstract_html_url = content;

		} else if (tag.equals(MetaTags.CITATION_COLLECTION_ID)) {
			publicationParsingMetaModel.citation_collection_id = content;
			
		} else if (tag.equals(MetaTags.CITATION_LANGUAGE)) {
			publicationParsingMetaModel.citation_language = content;
			
		}  else if (tag.equals(MetaTags.CITATION_KEYWORDS)) {
			publicationParsingMetaModel.citation_keywords.add(content);	
			
		} else if (tag.equals(MetaTags.CITATION_PMID)) {
			publicationParsingMetaModel.citation_pmid = content;
			
		} else if (tag.equals(MetaTags.CITATION_CONFERENCE_TITLE)) {
			publicationParsingMetaModel.citation_conference_title = content;

		} else if (tag.equals(MetaTags.CITATION_FULLTEXT_HTML_URL)) {
			publicationParsingMetaModel.citation_fulltext_html_url = content;

		} else if (tag.equals(MetaTags.CITATION_AUTHOR)) {
			publicationParsingMetaModel.citation_author.add(content);

		} else if (tag.equals(MetaTags.CITATION_AUTHORS)) {
			publicationParsingMetaModel.citation_authors = content;

		} else if (tag.equals(MetaTags.CITATION_DATE)) {
			publicationParsingMetaModel.citation_date = content;

		} else if (tag.equals(MetaTags.CITATION_DOI)) {
			publicationParsingMetaModel.citation_doi = content;

		} else if (tag.equals(MetaTags.CITATION_JOURNAL_TITLE)) {
			publicationParsingMetaModel.citation_journal_title = content;

		} else if (tag.equals(MetaTags.CITATION_JOURNAL_ABBREV)) {
			publicationParsingMetaModel.citation_journal_abbrev = content;

		} else if (tag.equals(MetaTags.CITATION_ID)) {
			publicationParsingMetaModel.citation_id = content;
		
		} else if (tag.equals(MetaTags.CITATION_ISSN)) {
			publicationParsingMetaModel.citation_issn.add(content);

		} else if (tag.equals(MetaTags.CITATION_ISBN)) {
			publicationParsingMetaModel.citation_isbn = content;

		} else if (tag.equals(MetaTags.CITATION_VOLUME)) {
			publicationParsingMetaModel.citation_volume = content;

		} else if (tag.equals(MetaTags.CITATION_ISSUE)) {
			publicationParsingMetaModel.citation_issue = content;

		} else if (tag.equals(MetaTags.CITATION_FIRSTPAGE)) {
			publicationParsingMetaModel.citation_firstpage = content;

		} else if (tag.equals(MetaTags.CITATION_LASTPAGE)) {
			publicationParsingMetaModel.citation_lastpage = content;

		} else if (tag.equals(MetaTags.CITATION_PUBLICATION_DATE)) {
			publicationParsingMetaModel.citation_publication_date = content;
			
		} else if (tag.equals(MetaTags.CITATION_PUBLISHER)) {
			publicationParsingMetaModel.citation_publisher = content;

		} else if (tag.equals(MetaTags.CITATION_PDF_URL)) {
			publicationParsingMetaModel.citation_pdf_url = content;

		} else if (tag.equals(MetaTags.CITATION_ONLINE_DATE)) {
			publicationParsingMetaModel.citation_online_date = content;

		} else if (tag.equals(MetaTags.CITATION_MJID)) {
			publicationParsingMetaModel.citation_mjid = content;

		} else if (tag.equals(MetaTags.CITATION_PUBLIC_URL)) {
			publicationParsingMetaModel.citation_public_url= content;
			
		} else if (tag.equals(MetaTags.CITATION_DISSERTATION_INSTITUTION)) {
			publicationParsingMetaModel.citation_dissertation_institution = content;

		} else if (tag.equals(MetaTags.CITATION_TECHNICAL_REPORT_INSTITUTION)) {
			publicationParsingMetaModel.citation_technical_report_institution = content;

		} else if (tag.equals(MetaTags.CITATION_TECHNICAL_REPORT_NUMBER)) {
			publicationParsingMetaModel.citation_technical_report_number = content;

		} else if (tag.equals(MetaTags.CITATION_VOLUME)) {
			publicationParsingMetaModel.citation_volume = content;

		} else if (tag.equals(MetaTags.CITATION_SECTION)) {
			publicationParsingMetaModel.citation_section = content;

		}

		//////////////////////////////////////////////////////////////////////
		// DC Tags => dublin core.
		//////////////////////////////////////////////////////////////////////
		else if (tag.equals(MetaTags.DC_TITLE)) {
			publicationParsingMetaModel.dcTitle = content;

		} else if (tag.equals(MetaTags.DC_CREATOR)) {
			publicationParsingMetaModel.dcCreator.add(content);

		} else if (tag.equals(MetaTags.DC_SUBJECT)) {
			publicationParsingMetaModel.dcSubject = content;

		} else if (tag.equals(MetaTags.DC_SOURCE)) {
			publicationParsingMetaModel.dcSource = content;

		} else if (tag.equals(MetaTags.DC_RELATION)) {
			publicationParsingMetaModel.dcRelation = content;

		} else if (tag.equals(MetaTags.DC_DESCRIPTION)) {
			publicationParsingMetaModel.dcDescription = content;

		} else if (tag.equals(MetaTags.DC_PUBLISHER)) {
			publicationParsingMetaModel.dcPublisher = content;

		} else if (tag.equals(MetaTags.DC_CONTRIBUTOR)) {
			publicationParsingMetaModel.dcContributor.add(content);

		} else if (tag.equals(MetaTags.DC_DATE)) {
			publicationParsingMetaModel.dcDate = content;
			
		} else if (tag.equals(MetaTags.DC_ISSUED)) {
			publicationParsingMetaModel.dcIssued = content;

		} else if (tag.equals(MetaTags.DC_IDENTIFIER)) {
			publicationParsingMetaModel.dcIdentifier = content;

		} else if (tag.equals(MetaTags.DC_LANGUAGE)) {
			publicationParsingMetaModel.dcLanguage = content;
		} else if (tag.equals(MetaTags.DC_RIGHTS)) {
			publicationParsingMetaModel.dcRights = content;

		} else if (tag.equals(MetaTags.DC_RIGHTS_AGENT)) {
			publicationParsingMetaModel.dcRightsAgent = content;

		} else if (tag.equals(MetaTags.DC_COVERAGE)) {
			publicationParsingMetaModel.dcCoverage = content;

		} else if (tag.equals(MetaTags.DC_FORMAT)) {
			publicationParsingMetaModel.dcFormat = content;

		} else if (tag.equals(MetaTags.DC_TYPE)) {
			publicationParsingMetaModel.dcType = content;

		//////////////////////////////////////////////////////////////////////
		// PRISM Tags.
		//////////////////////////////////////////////////////////////////////
		} else if (tag.equals(MetaTags.PRISM_COPYRIGHT)) {
			publicationParsingMetaModel.prismCopyright = content;

		} else if (tag.equals(MetaTags.PRISM_RIGHTS_AGENT)) {
			publicationParsingMetaModel.prismRightsAgent = content;

		} else if (tag.equals(MetaTags.PRISM_VOLUME)) {
			publicationParsingMetaModel.prismVolume = content;

		} else if (tag.equals(MetaTags.PRISM_PUBLICATION_DATE)) {
			publicationParsingMetaModel.prismPublicationDate = content;

		} else if (tag.equals(MetaTags.PRISM_ISSUE)) {
			publicationParsingMetaModel.prismIssue = content;

		} else if (tag.equals(MetaTags.PRISM_NUMBER)) {
			publicationParsingMetaModel.prismNumber = content;

		} else if (tag.equals(MetaTags.PRISM_SECTION)) {
			publicationParsingMetaModel.prismSection = content;

		} else if (tag.equals(MetaTags.PRISM_STARTING_PAGE)) {
			publicationParsingMetaModel.prismStartingPage = content;

		} else if (tag.equals(MetaTags.PRISM_ENDING_PAGE)) {
			publicationParsingMetaModel.prismEndingPage = content;

		} else if (tag.equals(MetaTags.PRISM_PUBLICATION_NAME)) {
			publicationParsingMetaModel.prismPublicationName = content;

		} else if (tag.equals(MetaTags.PRISM_ISSN)) {
			publicationParsingMetaModel.prismIssn = content;

		} else if (tag.equals(MetaTags.PRISM_E_ISSN)) {
			publicationParsingMetaModel.prismEIssn = content;

		} else if (tag.equals(MetaTags.PRISM_STARTING_PAGE)) {
			publicationParsingMetaModel.prismStartingPage = content;

		}

	}

}
